"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var base_converter_1 = require("./base-converter");
var router_1 = require("../../router");
var kamboja_core_1 = require("kamboja-core");
var DecoratedValueConverter = /** @class */ (function (_super) {
    tslib_1.__extends(DecoratedValueConverter, _super);
    function DecoratedValueConverter() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DecoratedValueConverter.prototype.convert = function (meta, value) {
        var _this = this;
        var decorators = meta.decorators && meta.decorators.filter(function (x) { return _this.filterTypes(x); });
        if (!decorators || decorators.length == 0)
            return this.next();
        if (typeof value == "undefined" || value == null)
            return this.exit(value);
        var type = this.getParameter(decorators[0]);
        if (typeof value == "object" && !Array.isArray(value))
            throw new Error("Expected parameter type of [@type('" + type + "') " + meta.name + "] but got object in " + router_1.getRouteDetail(this.routeInfo));
        switch (type) {
            case "string":
                return this.exit(value.toString());
            case "number":
                return this.exit(parseInt(value));
            case "boolean":
                return this.exit(value.toLowerCase() === "true");
        }
        //all array types (string[], number[], boolean[]) is passed to the default conversion
        return this.next();
    };
    DecoratedValueConverter.prototype.filterTypes = function (x) {
        var _this = this;
        return (x.name == "type" &&
            kamboja_core_1.ValidationTypesAccepted.some(function (y) { return _this.getParameter(x) == y; }));
    };
    DecoratedValueConverter.prototype.getParameter = function (x) {
        return x.parameters[0].value
            .toString()
            .toLowerCase();
    };
    return DecoratedValueConverter;
}(base_converter_1.BaseConverter));
exports.DecoratedValueConverter = DecoratedValueConverter;
