"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var factory_1 = require("../factory");
var ControllerInvocation = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerInvocation, _super);
    function ControllerInvocation(middlewares, controllerInfo, facade) {
        var _this = _super.call(this) || this;
        _this.middlewares = middlewares;
        _this.controllerInfo = controllerInfo;
        _this.facade = facade;
        return _this;
    }
    ControllerInvocation.prototype.setError = function (error) {
        this.error = error;
    };
    ControllerInvocation.prototype.proceed = function () {
        if (this.error)
            throw this.error;
        var controller = factory_1.ControllerFactory.resolve(this.controllerInfo, this.facade.dependencyResolver);
        controller.context = this.context;
        controller.invocation = this;
        var method = controller[this.controllerInfo.methodMetaData.name];
        var result;
        if (this.facade.autoValidation && this.controllerInfo.methodMetaData.parameters.length > 0 && !controller.validator.isValid())
            result = new Core.ActionResult(controller.validator.getValidationErrors(), 400, "application/json");
        else
            result = method.apply(controller, this.parameters);
        return this.createResult(result);
    };
    ControllerInvocation.prototype.createResult = function (result) {
        var _this = this;
        return Promise.resolve(result)
            .then(function (awaitedResult) {
            if (awaitedResult instanceof Core.ActionResult)
                return awaitedResult;
            if (_this.controllerInfo.classMetaData.baseClass == "ApiController") {
                return new Core.ActionResult(awaitedResult, 200, "application/json");
            }
            return new Core.ActionResult(awaitedResult, 200, "text/html");
        });
    };
    return ControllerInvocation;
}(Core.Invocation));
exports.ControllerInvocation = ControllerInvocation;
