"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var missing_action_parameter_1 = require("./missing-action-parameter");
var missing_route_parameter_1 = require("./missing-route-parameter");
var non_associated_parameter_1 = require("./non-associated-parameter");
var duplicate_route_1 = require("./duplicate-route");
var conflict_internal_decorator_1 = require("./conflict-internal-decorator");
var convention_fail_1 = require("./convention-fail");
var class_not_controller_1 = require("./class-not-controller");
var class_not_exported_1 = require("./class-not-exported");
var reserved_word_used_1 = require("./reserved-word-used");
var event_with_query_parameters_1 = require("./event-with-query-parameters");
var RouteAnalyzer = /** @class */ (function () {
    function RouteAnalyzer(routes) {
        this.routes = routes;
        this.commands = [
            new missing_action_parameter_1.MissingActionParameterAnalyzer(),
            new missing_route_parameter_1.MissingRouteParameterAnalyzer(),
            new non_associated_parameter_1.UnassociatedParameterAnalyzer(),
            new duplicate_route_1.DuplicateRouteAnalyzer(),
            new conflict_internal_decorator_1.ConflictInternalDecoratorAnalyzer(),
            new convention_fail_1.ConventionFailDecoratorAnalyzer(),
            new class_not_controller_1.ClassNotInherritedFromControllerAnalyzer(),
            new class_not_exported_1.ClassNotExportedControllerAnalyzer(),
            new reserved_word_used_1.ReservedWordUsedAnalyzer(),
            new event_with_query_parameters_1.EventWithQueryParameters()
        ];
    }
    RouteAnalyzer.prototype.analyse = function () {
        var result = [];
        for (var _i = 0, _a = this.routes; _i < _a.length; _i++) {
            var route = _a[_i];
            for (var _b = 0, _c = this.commands; _b < _c.length; _b++) {
                var command = _c[_b];
                var analysis = command.analyse(route);
                if (analysis)
                    result = result.concat(analysis);
            }
        }
        return result;
    };
    return RouteAnalyzer;
}());
exports.RouteAnalyzer = RouteAnalyzer;
