"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var qualified_name_1 = require("../resolver/qualified-name");
var Fs = require("fs");
var Path = require("path");
var Kecubung = require("kecubung");
var Babylon = require("babylon");
function flatten(metaList, fileName) {
    var result = [];
    metaList.forEach(function (x) {
        switch (x.type) {
            case "Module":
                var file_1 = x;
                var clazz = flatten(file_1.children, fileName);
                if (clazz && clazz.length > 0) {
                    clazz.forEach(function (cls) {
                        cls.qualifiedClassName = file_1.name + "." + cls.qualifiedClassName;
                    });
                    result = result.concat(clazz);
                }
                break;
            case "Class":
                var curClass = x;
                curClass.qualifiedClassName = curClass.name + ", " + fileName;
                result.push(curClass);
                break;
        }
    });
    return result;
}
var MetaDataLoader = /** @class */ (function () {
    function MetaDataLoader(idResolver, pathResolver) {
        this.idResolver = idResolver;
        this.pathResolver = pathResolver;
        this.flatStorage = {};
        this.storage = {};
    }
    MetaDataLoader.prototype.load = function (path, category) {
        var files;
        if (typeof path == "string")
            files = this.getFilePaths([path], category);
        else
            files = this.getFilePaths(path, category);
        var flatResult = [];
        var result = [];
        for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
            var file = files_1[_i];
            var code = Fs.readFileSync(file).toString();
            var ast = Babylon.parse(code);
            var fileName = this.pathResolver.normalize(file);
            var meta = Kecubung.transform("ASTree", ast, fileName);
            flatResult = flatResult.concat(flatten(meta.children, meta.name));
            result.push(meta);
        }
        this.flatStorage[category] = flatResult;
        this.storage[category] = result;
    };
    MetaDataLoader.prototype.getFilePaths = function (paths, category) {
        var result = [];
        var _loop_1 = function (path) {
            var fileDirectory = this_1.pathResolver.resolve(path);
            if (!Fs.existsSync(fileDirectory) && category == "Controller")
                throw Error("Directory not found [" + path + "]");
            if (!Fs.existsSync(fileDirectory) && category == "Model")
                return "continue";
            var resultPaths = Fs.readdirSync(fileDirectory)
                .filter(function (x) { return Path.extname(x) == ".js"; })
                .map(function (x) { return Path.join(fileDirectory, x); });
            result = result.concat(resultPaths);
        };
        var this_1 = this;
        for (var _i = 0, paths_1 = paths; _i < paths_1.length; _i++) {
            var path = paths_1[_i];
            _loop_1(path);
        }
        return result;
    };
    MetaDataLoader.prototype.getFiles = function (category) {
        return this.storage[category];
    };
    MetaDataLoader.prototype.getClasses = function (category) {
        return this.flatStorage[category];
    };
    MetaDataLoader.prototype.get = function (classId) {
        var _this = this;
        var request = new qualified_name_1.QualifiedName(classId, this.pathResolver);
        for (var key in this.flatStorage) {
            var result = this.flatStorage[key].filter(function (x) {
                var qualified = new qualified_name_1.QualifiedName(x.qualifiedClassName, _this.pathResolver);
                return request.equals(qualified);
            });
            if (result.length > 0)
                return result[0];
        }
    };
    return MetaDataLoader;
}());
exports.MetaDataLoader = MetaDataLoader;
