"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var transformer_base_1 = require("./transformer-base");
var ApiConventionTransformer = /** @class */ (function (_super) {
    tslib_1.__extends(ApiConventionTransformer, _super);
    function ApiConventionTransformer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.conventions = ["get", "list", "add", "replace", "modify", "delete"];
        return _this;
    }
    ApiConventionTransformer.prototype.transform = function (meta, parent, prevResult) {
        if (prevResult) {
            return this.next(prevResult);
        }
        if (this.conventions.some(function (x) { return x == meta.name; })) {
            //if it doesn't contain parameter pass to other transformer
            if (meta.parameters.length < 1) {
                return this.next({
                    analysis: [Core.RouteAnalysisCode.ConventionFail],
                    initiator: "ApiConvention",
                    methodMetaData: meta,
                    overrideRequest: Core.OverrideRequest.Route,
                    //GET by design
                    httpMethod: "GET"
                });
            }
            switch (meta.name) {
                case "list":
                    return this.noParam(meta, parent, "GET");
                case "get":
                    return this.singleParam(meta, parent, "GET");
                case "delete":
                    return this.singleParam(meta, parent, "DELETE");
                case "replace":
                    return this.singleParam(meta, parent, "PUT");
                case "modify":
                    return this.singleParam(meta, parent, "PATCH");
                case "add":
                    return this.noParam(meta, parent, "POST");
            }
        }
        return this.next();
    };
    ApiConventionTransformer.prototype.singleParam = function (meta, parent, method) {
        var path = "/:" + meta.parameters[0].name;
        //auto assigned required validation on id
        if (!meta.parameters[0].decorators)
            meta.parameters[0].decorators = [];
        if (!meta.parameters[0].decorators.some(function (x) { return x.name == "required"; })) {
            meta.parameters[0].decorators.push({
                type: 'Decorator',
                name: 'required',
                analysis: 1,
                location: { start: 1124, end: 1164 },
                parameters: []
            });
        }
        return this.exit({
            httpMethod: method,
            initiator: "ApiConvention",
            methodMetaData: meta,
            route: parent + path,
            methodPath: path
        });
    };
    ApiConventionTransformer.prototype.noParam = function (meta, parent, method) {
        return this.exit({
            httpMethod: method,
            initiator: "ApiConvention",
            methodMetaData: meta,
            route: parent,
            methodPath: "/"
        });
    };
    tslib_1.__decorate([
        transformer_base_1.when("Method")
    ], ApiConventionTransformer.prototype, "transform", null);
    return ApiConventionTransformer;
}(transformer_base_1.TransformerBase));
exports.ApiConventionTransformer = ApiConventionTransformer;
