"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var transformer_base_1 = require("./transformer-base");
var controller_1 = require("./controller");
var ControllerWithDecorator = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerWithDecorator, _super);
    function ControllerWithDecorator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControllerWithDecorator.prototype.transform = function (meta, parent, prevResult) {
        var _this = this;
        var checkResult = controller_1.checkForAnalysis(meta);
        if (checkResult)
            return this.exit(checkResult);
        this.transformers = controller_1.getTransformers(meta);
        if (meta.decorators && meta.decorators.some(function (x) { return x.name == "root"; })) {
            var decorators = meta.decorators.filter(function (x) { return x.name == "root"; });
            if (decorators.length > 1) {
                return this.exit({
                    analysis: [Core.RouteAnalysisCode.DuplicateRoutes],
                    qualifiedClassName: meta.name,
                    initiator: "ControllerWithDecorator",
                    classMetaData: meta
                });
            }
            var route_1 = decorators[0].parameters[0].value;
            if (route_1.charAt(0) != "/") {
                if (!parent)
                    parent = "";
                route_1 = parent + "/" + route_1;
            }
            this.autoAssignedValidator(meta, route_1);
            var result = this.transformChildren(meta.methods, route_1);
            result.forEach(function (x) {
                _this.checkUnassociatedParameter(x, route_1);
                x.qualifiedClassName = meta.name;
                x.classMetaData = meta;
                if (!x.collaborator)
                    x.collaborator = [];
                x.collaborator.push("ControllerWithDecorator");
            });
            return this.exit(result);
        }
        else
            return this.next();
    };
    ControllerWithDecorator.prototype.checkUnassociatedParameter = function (info, route) {
        var param = route.split("/").filter(function (x) { return x.charAt(0) == ":"; })
            .map(function (x) { return x.substring(1); });
        var match = info.methodMetaData.parameters.filter(function (x) { return param.some(function (y) { return x.name == y; }); });
        if (match.length != param.length) {
            if (!info.analysis)
                info.analysis = [];
            if (!info.analysis.some(function (x) { return x == Core.RouteAnalysisCode.UnAssociatedParameters; }))
                info.analysis.push(Core.RouteAnalysisCode.UnAssociatedParameters);
        }
    };
    ControllerWithDecorator.prototype.autoAssignedValidator = function (meta, route) {
        var param = route.split("/").filter(function (x) { return x.charAt(0) == ":"; })
            .map(function (x) { return x.substring(1); });
        meta.methods.forEach(function (method) {
            var getAction = meta.methods.filter(function (x) { return x.name == method.name; })[0];
            var parameters = getAction.parameters.filter(function (x) { return param.some(function (y) { return y == x.name; }); });
            parameters.forEach(function (x) {
                if (!x.decorators)
                    x.decorators = [];
                if (!x.decorators.some(function (x) { return x.name == "required"; })) {
                    x.decorators.push({
                        type: 'Decorator',
                        name: 'required',
                        analysis: 1,
                        location: { start: 1124, end: 1164 },
                        parameters: []
                    });
                }
            });
        });
    };
    tslib_1.__decorate([
        transformer_base_1.when("Class")
    ], ControllerWithDecorator.prototype, "transform", null);
    return ControllerWithDecorator;
}(transformer_base_1.TransformerBase));
exports.ControllerWithDecorator = ControllerWithDecorator;
