"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Kecubung = require("kecubung");
var Core = require("kamboja-core");
var transformer_base_1 = require("./transformer-base");
var api_convention_1 = require("./api-convention");
var default_action_1 = require("./default-action");
var http_decorator_1 = require("./http-decorator");
var internal_decorator_1 = require("./internal-decorator");
var event_decorator_1 = require("./event-decorator");
function getName(meta) {
    var ctlLocation = meta.name.toLowerCase().lastIndexOf("controller");
    if (ctlLocation > 0) {
        return meta.name.substr(0, ctlLocation).toLocaleLowerCase();
    }
    else {
        return meta.name.toLocaleLowerCase();
    }
}
function getTransformers(meta) {
    //highest priority transformer should stay on top of another
    if (meta.baseClass == "ApiController") {
        return [
            new internal_decorator_1.InternalDecoratorTransformer(),
            new http_decorator_1.HttpDecoratorTransformer(),
            new event_decorator_1.EventDecoratorTransformer(),
            new api_convention_1.ApiConventionTransformer(),
            new default_action_1.DefaultActionTransformer()
        ];
    }
    else {
        return [
            new internal_decorator_1.InternalDecoratorTransformer(),
            new http_decorator_1.HttpDecoratorTransformer(),
            new event_decorator_1.EventDecoratorTransformer(),
            //new IndexActionTransformer(),
            new default_action_1.DefaultActionTransformer()
        ];
    }
}
exports.getTransformers = getTransformers;
function checkForAnalysis(meta) {
    //check if class inherrited from Controler or ApiController
    if (!meta.baseClass ||
        !(meta.baseClass == "Controller"
            || meta.baseClass == "ApiController"))
        return {
            analysis: [Core.RouteAnalysisCode.ClassNotInheritedFromController],
            qualifiedClassName: meta.name,
            initiator: "Controller",
            classMetaData: meta
        };
    //check if class is valid (exported)
    if (!Kecubung.flag(meta.analysis, Kecubung.AnalysisType.Valid))
        return {
            analysis: [Core.RouteAnalysisCode.ClassNotExported],
            qualifiedClassName: meta.name,
            initiator: "Controller",
            classMetaData: meta
        };
    return;
}
exports.checkForAnalysis = checkForAnalysis;
var ControllerTransformer = /** @class */ (function (_super) {
    tslib_1.__extends(ControllerTransformer, _super);
    function ControllerTransformer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ControllerTransformer.prototype.transform = function (meta, parent, prevResult) {
        var checkResult = checkForAnalysis(meta);
        if (checkResult)
            return this.exit(checkResult);
        this.transformers = getTransformers(meta);
        var name = getName(meta);
        if (!parent)
            parent = "";
        parent += "/" + name;
        var result = this.transformChildren(meta.methods, parent);
        result.forEach(function (x) {
            x.qualifiedClassName = meta.name;
            x.classMetaData = meta;
            if (!x.collaborator)
                x.collaborator = [];
            x.collaborator.push("Controller");
        });
        return this.exit(result);
    };
    tslib_1.__decorate([
        transformer_base_1.when("Class")
    ], ControllerTransformer.prototype, "transform", null);
    return ControllerTransformer;
}(transformer_base_1.TransformerBase));
exports.ControllerTransformer = ControllerTransformer;
