"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var transformer_base_1 = require("./transformer-base");
var EventDecoratorTransformer = /** @class */ (function (_super) {
    tslib_1.__extends(EventDecoratorTransformer, _super);
    function EventDecoratorTransformer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    EventDecoratorTransformer.prototype.transform = function (meta, parent, prevResult) {
        var _this = this;
        if (prevResult) {
            //too complex to handle,
            //just pass previous result
            return this.next(prevResult);
        }
        if (meta.decorators && meta.decorators.length > 0 && meta.decorators.some(function (x) { return x.name == "on"; })) {
            var decorators = meta.decorators.filter(function (x) { return x.name == "on"; });
            var infos = decorators.map(function (decorator) {
                return _this.createInfo(meta, decorator);
            });
            return this.exit(infos);
        }
        else
            return this.next();
    };
    EventDecoratorTransformer.prototype.createInfo = function (meta, decorator) {
        var analysis = [];
        var method = "EVENT";
        var route = decorator.parameters[0].value;
        var routeAnalysis = this.checkIfQueryParameterNotAllowed(route, method);
        if (routeAnalysis)
            analysis.push(routeAnalysis);
        var info = {
            initiator: "HttpMethodDecorator",
            httpMethod: method,
            methodMetaData: meta,
            route: route
        };
        if (analysis.length > 0)
            info.analysis = analysis;
        return info;
    };
    EventDecoratorTransformer.prototype.checkIfQueryParameterNotAllowed = function (route, method) {
        var routeParameters = route.split("/").filter(function (x) { return x.charAt(0) == ":"; });
        if (routeParameters.length > 0)
            return Core.RouteAnalysisCode.QueryParameterNotAllowed;
    };
    tslib_1.__decorate([
        transformer_base_1.when("Method")
    ], EventDecoratorTransformer.prototype, "transform", null);
    return EventDecoratorTransformer;
}(transformer_base_1.TransformerBase));
exports.EventDecoratorTransformer = EventDecoratorTransformer;
