"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Core = require("kamboja-core");
var transformer_base_1 = require("./transformer-base");
var HttpDecoratorTransformer = /** @class */ (function (_super) {
    tslib_1.__extends(HttpDecoratorTransformer, _super);
    function HttpDecoratorTransformer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.decorators = ["get", "post", "put", "delete", "patch"];
        return _this;
    }
    HttpDecoratorTransformer.prototype.transform = function (meta, parent, prevResult) {
        var _this = this;
        if (prevResult) {
            //too complex to handle,
            //just pass previous result
            return this.next(prevResult);
        }
        if (meta.decorators && meta.decorators.length > 0) {
            var result = [];
            var decorators = meta.decorators.filter(function (x) { return _this.decorators.some(function (y) { return y == x.name; }); });
            //single method can be assigned with multiple route,
            //so the result is multiple RouteInfo
            for (var _i = 0, decorators_1 = decorators; _i < decorators_1.length; _i++) {
                var decorator = decorators_1[_i];
                var info = this.createInfo(meta, decorator, parent);
                result.push(info);
            }
            //pass to the default action generator to fill decorator without parameter
            return this.next(result.length > 0 ? result : undefined);
        }
        else
            return this.next();
    };
    HttpDecoratorTransformer.prototype.createInfo = function (meta, decorator, parent) {
        var method = decorator.name.toUpperCase();
        //if decorator doesn't contains parameter (url) then 
        //left the url empty and pass to the next transformer
        if (!decorator.parameters || decorator.parameters.length == 0) {
            return {
                initiator: "HttpMethodDecorator",
                httpMethod: method,
                methodMetaData: meta,
                overrideRequest: Core.OverrideRequest.Route
            };
        }
        else {
            var route = decorator.parameters[0].value;
            //if route is relative add with parent
            if (route.charAt(0) != "/")
                route = parent + "/" + route;
            var analysis = [];
            var routeAnalysis = this.checkMissingActionParameters(meta, route, method);
            if (routeAnalysis)
                analysis.push(routeAnalysis);
            routeAnalysis = this.checkMissingRouteParameters(meta, route, method);
            if (routeAnalysis)
                analysis.push(routeAnalysis);
            routeAnalysis = this.checkUnAssociatedParameters(meta, route, method);
            if (routeAnalysis)
                analysis.push(routeAnalysis);
            var result = {
                initiator: "HttpMethodDecorator",
                httpMethod: method,
                methodMetaData: meta,
                route: route,
            };
            if (analysis.length > 0)
                result.analysis = analysis;
            return result;
        }
    };
    HttpDecoratorTransformer.prototype.checkMissingActionParameters = function (meta, route, method) {
        //analyse if route contains parameter but method without parameter
        var routeParameters = route.split("/").filter(function (x) { return x.charAt(0) == ":"; });
        if (routeParameters.length > 0 && meta.parameters.length == 0) {
            return Core.RouteAnalysisCode.MissingActionParameters;
        }
    };
    HttpDecoratorTransformer.prototype.checkMissingRouteParameters = function (meta, route, method) {
        //analyse if method contains parameter but route without parameter
        //this check only work for GET method, because other method can pass a BODY to the parameter
        var routeParameters = route.split("/").filter(function (x) { return x.charAt(0) == ":"; });
        if (method == "GET" && routeParameters.length == 0 && meta.parameters.length > 0) {
            return Core.RouteAnalysisCode.MissingRouteParameters;
        }
    };
    HttpDecoratorTransformer.prototype.checkUnAssociatedParameters = function (meta, route, method) {
        //analyse if provided has associated parameter
        var parameters = meta.parameters.map(function (x) { return x.name; });
        var routeParameters = route.split("/").filter(function (x) { return x.charAt(0) == ":"; });
        var _loop_1 = function (x) {
            var parName = x.substring(1);
            if (!parameters.some(function (y) { return y == parName; })) {
                return { value: Core.RouteAnalysisCode.UnAssociatedParameters };
            }
        };
        for (var _i = 0, routeParameters_1 = routeParameters; _i < routeParameters_1.length; _i++) {
            var x = routeParameters_1[_i];
            var state_1 = _loop_1(x);
            if (typeof state_1 === "object")
                return state_1.value;
        }
    };
    tslib_1.__decorate([
        transformer_base_1.when("Method")
    ], HttpDecoratorTransformer.prototype, "transform", null);
    return HttpDecoratorTransformer;
}(transformer_base_1.TransformerBase));
exports.HttpDecoratorTransformer = HttpDecoratorTransformer;
