"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
var MetaDataKey = "kamboja:Validator";
function decoratorName(decoratorName) {
    return function (target, propertyKey, index) {
        Reflect.defineMetadata(MetaDataKey, decoratorName, target);
    };
}
exports.decoratorName = decoratorName;
function getDecoratorName(target) {
    return Reflect.getMetadata(MetaDataKey, target);
}
exports.getDecoratorName = getDecoratorName;
var ValidatorBase = /** @class */ (function () {
    function ValidatorBase() {
    }
    ValidatorBase.prototype.validate = function (args) {
        return undefined;
    };
    ValidatorBase.prototype.isEmpty = function (value) {
        return typeof value == "undefined"
            || value == null
            || (typeof value == "string" && value.trim() == "");
    };
    ValidatorBase.prototype.validateFields = function (arg) {
        if (arg.type == "PropertiesValidator") {
            var result = [];
            for (var _i = 0, _a = arg.classInfo.properties; _i < _a.length; _i++) {
                var property = _a[_i];
                if (arg.isArray) {
                    if (!Array.isArray(arg.classInstance))
                        return [{
                                field: arg.parentField,
                                message: "[" + arg.parentField + "] must be a type of Array"
                            }];
                    var instances = arg.classInstance;
                    for (var i = 0; i < instances.length; i++) {
                        var instance = instances[i];
                        var value = instance[property.name];
                        var valResult = this.useValidators({
                            decorators: property.decorators,
                            value: value,
                            field: property.name,
                            parentField: arg.parentField + "[" + i + "]",
                            classInfo: arg.classInfo
                        });
                        result = result.concat(valResult);
                    }
                }
                else {
                    var value = arg.classInstance[property.name];
                    var valResult = this.useValidators({
                        decorators: property.decorators,
                        value: value,
                        field: property.name,
                        parentField: arg.parentField,
                        classInfo: arg.classInfo
                    });
                    result = result.concat(valResult);
                }
            }
            return result.length == 0 ? undefined : result;
        }
        else {
            var result = [];
            var method = arg.classInfo.methods.filter(function (x) { return x.name == arg.methodName; })[0];
            for (var i = 0; i < method.parameters.length; i++) {
                var parameterMetadata = method.parameters[i];
                var value = arg.parameterValues[i];
                var valResult = this.useValidators({
                    decorators: parameterMetadata.decorators,
                    classInfo: arg.classInfo,
                    field: parameterMetadata.name,
                    parentField: arg.parentField,
                    value: value
                });
                if (valResult)
                    result = result.concat(valResult);
            }
            return result.length == 0 ? undefined : result;
        }
    };
    ValidatorBase.prototype.useValidators = function (arg) {
        var result = [];
        if (!arg.decorators)
            return;
        var _loop_1 = function (decorator) {
            var validators = this_1.validators.filter(function (x) { return getDecoratorName(x) == decorator.name; });
            for (var _i = 0, validators_1 = validators; _i < validators_1.length; _i++) {
                var validator_1 = validators_1[_i];
                var validationResult = validator_1.validate({
                    classInfo: arg.classInfo,
                    decoratorArgs: decorator.parameters,
                    field: arg.field,
                    parentField: arg.parentField ? "" + arg.parentField : arg.parentField,
                    value: arg.value
                });
                if (validationResult)
                    result = result.concat(validationResult);
            }
        };
        var this_1 = this;
        for (var _i = 0, _a = arg.decorators; _i < _a.length; _i++) {
            var decorator = _a[_i];
            _loop_1(decorator);
        }
        return result;
    };
    return ValidatorBase;
}());
exports.ValidatorBase = ValidatorBase;
