"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var baseclasses_1 = require("./baseclasses");
var RangeValidator = /** @class */ (function (_super) {
    tslib_1.__extends(RangeValidator, _super);
    function RangeValidator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RangeValidator.prototype.validate = function (args) {
        if (this.isEmpty(args.value))
            return;
        var fieldName = args.parentField ? args.parentField + "." + args.field : args.field;
        var minValue = undefined, maxValue = undefined, customMessage = undefined;
        minValue = args.decoratorArgs[0].value;
        if (args.decoratorArgs[1])
            maxValue = args.decoratorArgs[1].value;
        if (args.decoratorArgs[2])
            customMessage = args.decoratorArgs[2].value;
        if (typeof args.value == "number") {
            var numericValue = args.value;
            if (numericValue < minValue) {
                return [{
                        field: fieldName,
                        message: customMessage || "[" + args.field + "] must be greater than " + minValue
                    }];
            }
            if (maxValue && numericValue > maxValue) {
                return [{
                        field: fieldName,
                        message: customMessage || "[" + args.field + "] must be less than " + maxValue
                    }];
            }
        }
        else if (typeof args.value == "string") {
            var length_1 = args.value.length;
            if (length_1 < minValue) {
                return [{
                        field: fieldName,
                        message: customMessage || "[" + args.field + "] length must be more than " + minValue + " characters"
                    }];
            }
            if (maxValue && length_1 > maxValue) {
                return [{
                        field: fieldName,
                        message: customMessage || "[" + args.field + "] length must be less than " + maxValue + " characters"
                    }];
            }
        }
    };
    tslib_1.__decorate([
        baseclasses_1.decoratorName("range")
    ], RangeValidator.prototype, "validate", null);
    return RangeValidator;
}(baseclasses_1.ValidatorBase));
exports.RangeValidator = RangeValidator;
