"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var baseclasses_1 = require("./baseclasses");
var kamboja_core_1 = require("kamboja-core");
var qualified_name_1 = require("../resolver/qualified-name");
var TypeValidator = /** @class */ (function (_super) {
    tslib_1.__extends(TypeValidator, _super);
    function TypeValidator(storage) {
        var _this = _super.call(this) || this;
        _this.storage = storage;
        return _this;
    }
    TypeValidator.prototype.validate = function (args) {
        var FIELD_NAME = args.parentField ? args.parentField + "." + args.field : args.field;
        if (this.isEmpty(args.value))
            return;
        if (!args.decoratorArgs)
            return;
        var decoratorArg = args.decoratorArgs[0];
        if (this.isEmpty(decoratorArg.value))
            throw new Error("Qualified class name should be specified in @type in [" + args.classInfo.name + "]");
        if (kamboja_core_1.ValidationTypesAccepted.some(function (x) { return x == decoratorArg.value.toLowerCase(); }))
            return;
        var qualified = new qualified_name_1.QualifiedName(decoratorArg.value, this.storage.pathResolver);
        if (!qualified.isValid())
            throw new Error("Invalid qualified class name [" + decoratorArg.value + "] in @type decorator in [" + args.classInfo.name + "]");
        var clazz = this.storage.get(qualified.qualifiedName);
        if (!clazz)
            throw new Error("Class [" + decoratorArg.value + "] in @type is no found in [" + args.classInfo.name + "]");
        return this.validateFields({
            type: "PropertiesValidator",
            classInfo: clazz,
            parentField: args.parentField ? args.parentField + "." + args.field : args.field,
            classInstance: args.value,
            isArray: qualified.isArray()
        });
    };
    tslib_1.__decorate([
        baseclasses_1.decoratorName("type")
    ], TypeValidator.prototype, "validate", null);
    return TypeValidator;
}(baseclasses_1.ValidatorBase));
exports.TypeValidator = TypeValidator;
