"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var baseclasses_1 = require("./baseclasses");
var type_validator_1 = require("./type-validator");
var required_validator_1 = require("./required-validator");
var email_validator_1 = require("./email-validator");
var range_validator_1 = require("./range-validator");
var kamboja_1 = require("../kamboja");
var ValidatorImpl = /** @class */ (function (_super) {
    tslib_1.__extends(ValidatorImpl, _super);
    function ValidatorImpl(metaDataStorage, validators) {
        var _this = _super.call(this) || this;
        _this.metaDataStorage = metaDataStorage;
        _this.errors = [];
        if (!validators)
            validators = [];
        _this.validators = validators.slice();
        var typeValidator = new type_validator_1.TypeValidator(metaDataStorage);
        typeValidator.validators = _this.validators;
        _this.validators.push(typeValidator);
        _this.validators.push(new email_validator_1.EmailValidator());
        _this.validators.push(new range_validator_1.RangeValidator());
        _this.validators.push(new required_validator_1.RequiredValidator());
        return _this;
    }
    ValidatorImpl.create = function () {
        var facade = kamboja_1.Kamboja.getFacade();
        return new ValidatorImpl(facade.metaDataStorage, facade.validators);
    };
    ValidatorImpl.prototype.setValue = function (parameters, meta, methodName) {
        this.parameters = parameters;
        this.classMetaData = meta;
        this.methodName = methodName;
    };
    ValidatorImpl.prototype.isValid = function () {
        var result = this.validateFields({
            type: "ParametersValidator",
            classInfo: this.classMetaData,
            methodName: this.methodName,
            parameterValues: this.parameters,
        });
        if (result)
            this.errors = result;
        return this.errors.length == 0;
    };
    ValidatorImpl.prototype.getValidationErrors = function () {
        return this.errors.length > 0 ? this.errors : undefined;
    };
    return ValidatorImpl;
}(baseclasses_1.ValidatorBase));
exports.ValidatorImpl = ValidatorImpl;
